*----------------------------------------------------------------------------
*
*   Program Name: QUERY.PRG         Copyright: none
*   Date Created: 03/18/92           Language: Clipper 5.0
*   Time Created: 09:39:20             Author: Ed Phillips
*    Description: Query Maintenance modules
*----------------------------------------------------------------------------
#include "Set.ch"
#include "Inkey.ch"

#define FLDNAME 1
#define FLDTYPE 2
#define FLDLEN  3
#define FLDDEC  4

FUNCTION BldQuery(cGroup)
   FIELD Qname, Quser_id, Qfilter, QGroup, Qnum, Group, Query
   MEMVAR getlist, c_field, ex_flg, uid, cQueryName, ok

   LOCAL i := 1, nSel := 1, nRow := 5, connector, working, op
   LOCAL savscrn, cond := ' ', prom1, mcqnum, nCol := 38, cTit, lAdded
   LOCAL oldarea := Select(), oldcolor := Setcolor(), savewin, tmpclr
   LOCAL oldscore := Set(_SET_SCOREBOARD)

   PRIVATE xValue, aPtr := 1, mcQuery
   PRIVATE flist := {}, aPicks := {}

   IF Type('uid') != 'C'
      uid := Space(1)                       && create uid if needed
   ENDIF                                            && IF Type('uid') = 'U'

   IF !File('Qfield.dbf')
      RETURN cond
   ENDIF

   IF !File('Queries.dbf')
      BQFile()
   ENDIF                                            && IF !File('Query.dbf')

   USE Qfield ALIAS Qfield NEW

   IF ! Empty(cGroup)
      SET FILTER TO Trim(Group) == cGroup
   ENDIF                                         && IF ! Empty(cGroup)

   GO TOP
   WHILE ! Eof()
      IF Field->Query
         aAdd(aPicks, FIELD->Field_desc)
         aAdd(flist, { FIELD->Field_name, FIELD->Field_type, FIELD->Field_len, FIELD->Field_dec})
      ENDIF                                      && IF Field->Query

      SKIP
   END

   USE
   Setcolor(c_field)
   CLEAR

   cTit = ' Build a new query '
   @ 1,0 TO 23,79 DOUBLE
   @ 1,Centr(cTit) SAY cTit

   ok = .t.
   working := If(Len(aPicks) == 0, .f., .t.)
   @ 17,8 TO 22,72
   @ 17,31 SAY ' Query Definition '
   Scroll(18,9,21,71,0)

   WHILE working
      connector := xValue := ''

      *---------------------
      * Get the "field name"
      *---------------------
      nSel = PickList(aPicks, "Select a Field", 33, 3, 1, nSel, 14)
      @ 24,0 CLEAR

      IF nSel = 0
         EXIT
      ENDIF                                         && IF nSel = 0

      BEGIN SEQUENCE

         *-------------------
         * Get the "operator"
         *-------------------
         savscrn = Savescreen()
         op = OpBox(flist[nSel,FLDTYPE])

         IF Empty(op)
            ok = .f.
            working = .f.
            BREAK
         ENDIF                                      && IF Empty(op)

         *-----------------------------
         * Get the "value" of the field
         *-----------------------------
         ValInit('xValue', flist, nSel)

         tmpclr := Setcolor(M->c_pop)
         @ 7,21 TO 9,59
         Scroll(8,22,8,58,0)
         Shadow(7,21,9,59)

         Set(_SET_SCOREBOARD, .f.)

         @ 7,35 SAY ' Enter a Value '
         @ 8,23 GET xValue

         IF flist[nSel,FLDTYPE] = 'L'
            CLEAR GETS
         ELSE
            READ
         ENDIF
         Set(_SET_SCOREBOARD, oldscore)
         Setcolor(tmpclr)

         IF Empty(xValue)
            ok = .f.
            working := .f.
            BREAK
         ENDIF

         *----------------
         * Build condition
         *----------------
         cond += Condition(op, flist[nSel,FLDNAME], xValue, flist[nSel,FLDTYPE])
         Restscreen(,,,,savscrn)
         Saymemo(cond,18,10,60,4)

         *----------------------------
         * Get the logical "connector"
         *----------------------------
         connector = AndOrExit(5, 42)

         IF Upper(connector) = 'DONE'
            working = .f.
            BREAK
         ENDIF                                         && IF Upper(connector) = 'DONE'

         cond += ' .'+ connector + '.  '      && add connector to condition
      END
   ENDDO                                            && DO WHILE .T.

   IF ! Empty(cond) .AND. ok
      Alert1()
      IF Sayconfirm('Do you want to save this query? (Y/N)')
         mcQuery = Space(30)
         prom1 = 'Enter Query Name: '
         @ 24,Centr(prom1,Len(mcQuery)) SAY prom1 GET mcQuery
         READ
         IF ! Empty(mcQuery)
            ex_flg = .f.
            USE Queries ALIAS Queries NEW
            GO BOTTOM
            mcqnum = Strzero(Val(Qnum)+1,7,0)
            IF ! ex_flg
               APPEND BLANK

               QName := Trim(mcQuery)
               QFilter := AllTrim(cond)
               Qnum := mcqnum
               Quser_id := uid
            ENDIF
            USE
         ENDIF                                      && IF ! Empty(mcQuery)
      ENDIF                                         && IF Sayconfirm('Do you want to save this query? (Y/N')
   ELSE
      cond = ' '
   ENDIF                                            && IF ! Empty(cond)
   SELECT (oldarea)
   Setcolor(oldcolor)
RETURN (Upper(cond))

*----------------------------
*         Author: Ed Phillips
*   Date Created: 03/20/92
*----------------------------
FUNCTION OpBox(cFtype)
   LOCAL ret_val := Space(3), nSel := 0, nTop := 4, nLeft := 8, lErr
   LOCAL aOpList := { '=   - equals                  ',;
                      '<   - less than               ',;
                      '<=  - less than or equal to   ',;
                      '>   - greater than            ',;
                      '>=  - greater than or equal to',;
                      '#   - does not equal          ',;
                      '$   - contains                ',;
                      'NOT - not true                ' ;
                    }

   DO WHILE .T.
      lErr = .f.
      nSel = Picklist(aOpList, "Select an Operator", 30, nTop, nLeft,, 14)
      IF nSel != 0
         ret_val = Trim(Left(aOpList[nSel],3))
         DO CASE
            CASE cFtype $ 'DN'
               IF (ret_val $ '$  NOT')
                  lErr = .t.
               ENDIF                             && IF (ret_val $ '$  NOT')

            CASE cFtype = 'L'
               IF !(ret_val $ '=  NOT')
                  lErr = .t.
               ENDIF                             && IF !(ret_val $ '=  NOT')

            CASE cFtype $ 'CM'
               IF (ret_val = 'NOT')
                  lErr = .t.
               ENDIF                             && IF (ret_val = 'NOT')

            OTHERWISE
               EXIT
         ENDCASE                                 && DO CASE
         IF lErr
            Sayerr('Operator is not valid for field type')
            ret_val = Space(3)
         ENDIF                                   && IF lErr
      ENDIF                                         && IF nSel != 0
      IF ! lErr
         EXIT
      ENDIF                                      && IF ! lErr
   ENDDO                                         && DO WHILE .T.
RETURN (ret_val)

*----------------------------
*         Author: Ed Phillips
*   Date Created: 03/20/92
*----------------------------
FUNCTION AndOrExit(nRow, nCol, cColor)
   LOCAL ret_val := 'Done', item := 0, prompts := { 'AND', 'OR', 'Done' }
   LOCAL savscrn, oldcolor

   IF ValType(cColor) = 'U'
      cColor = M->c_pop
   ENDIF                                         && IF ValType(cColor) = 'U'
   oldcolor = Setcolor(cColor)
   savscrn = Savescreen(nRow-2,nCol, nRow+3, nCol+9)
   Scroll(nRow-2,nCol, nRow+2, nCol+7,0)
   @ nRow-2, nCol TO nRow+2, nCol+7
   Shadow(nRow-2, nCol, nRow+2, nCol+7)

   @ nRow-1, nCol+2 PROMPT 'AND'
   @ nRow,   nCol+2 PROMPT 'OR'
   @ nRow+1, nCol+2 PROMPT 'Done'

   MENU TO item

   IF item != 0
      ret_val = prompts[item]
   ENDIF                                         && IF item != 0
   Setcolor(oldcolor)
   Restscreen(nRow-2,nCol,nRow+3,nCol+9,savscrn)
RETURN (ret_val)


*----------------------------
*         Author: Ed Phillips
*   Date Created: 07/26/91
*----------------------------
FUNCTION ValInit(var,flist,nsel)
   LOCAL work, ftype, flen, fdec

   ftype := flist[nsel, FLDTYPE]
   flen := flist[nsel, FLDLEN]
   fdec := flist[nsel, FLDDEC]

   DO CASE
      CASE ftype = 'D'
         &var = CTOD([])

      CASE ftype = 'C' .OR. ftype = 'M'
         IF flen < 36 .AND. ftype != 'M'
            &var = Space(flen)
         ELSE
            &var = Space(35)
         ENDIF

      CASE ftype = 'N'
         work = Str(0, flen, fdec)       //Space(flen)
         &var = Val(work)

      CASE ftype = 'L'
         &var = 'True'

   ENDCASE
RETURN (NIL)

*----------------------------
*         Author: Ed Phillips
*   Date Created: 07/30/91
*----------------------------
FUNCTION Condition(op, cField, cVal, cType)
   LOCAL ret_val

   DO CASE
      CASE op = '$'
         ret_val = "'" + Trim(cVal) + "'" + ' $ ' + Trim(cField)

      CASE cType = 'D'
         ret_val = Trim(cField) + ' ' + op + ' Ctod("' + Dtoc(cVal) + '")'

      CASE cType = 'N'
         ret_val = Trim(cField) + ' ' + op + Ltrim(Str(cVal))

      CASE cType = 'C'
         ret_val = Trim(cField) + ' ' + op + ' "' + Trim(cVal) + '"'

      CASE cType = 'L'
         IF op = 'NOT'
            ret_val = '.NOT. ' + Trim(cField)
         ELSE
            ret_val = Trim(cField)
         ENDIF

      OTHERWISE
         ret_val = ' '

   ENDCASE
RETURN ( ret_val )

*----------------------------
*         Author: Ed Phillips
*   Date Created: 03/19/92
*----------------------------
FUNCTION BQFile()
   LOCAL  aDbf := {}

   aAdd( aDbf, {'Qname'   , 'C',  30, 0} )
   aAdd( aDbf, {'Quser_id', 'C',  20, 0} )
   aAdd( aDbf, {'Qfilter' , 'C', 250, 0} )
   aAdd( aDbf, {'Qgroup' ,  'C',  10, 0} )
   aAdd( aDbf, {'Qnum' ,    'C',   7, 0} )
   Dbcreate('Queries', aDbf)

RETURN (NIL)

*----------------------------
*         Author: Ed Phillips
*   Date Created: 03/20/92
*----------------------------
FUNCTION SelQuery(cColor, nTop, nLeft, nBrow)
   FIELD Qnum, Qname, Quser_id, Qfilter

   MEMVAR cQueryName
   LOCAL cond := Space(1), oldarea := Select(), nSel := 0, i := 1
   LOCAL cQnum, temp, oldcolor := Setcolor()
   LOCAL savscrn := Savescreen()

   IF ValType(cColor) = 'U'
      cColor := M->c_norm
   ENDIF                                         && IF ValType(cColor) = 'U'
   IF ValType(nTop) = 'U'
      nTop := 5
   ENDIF
   IF ValType(nLeft) = 'U'
      nLeft := 15
   ENDIF
   IF ValType(nBrow) = 'U'
      nBrow := 22
   ENDIF

   Setcolor(cColor)

   IF Select('QUERIES') == 0
      USE Queries ALIAS Queries NEW
   ENDIF                                         && IF Select('QUERY') = 0
   SELECT Queries

   PRIVATE aQueryList := {}
   GO TOP
   WHILE ! Eof()
      aAdd(aQueryList, Qname + Space(13) + Qnum)
      SKIP
   END

   nSel = Picklist(aQueryList, "Select a Query", 50, nTop, nLeft,, nBrow)
   IF nSel > 0
      cQnum = Right(aQueryList[nSel],7)

      LOCATE FOR Qnum == cQnum
      cond := EditQuery()
      IF SayConfirm('Save changes to Query? (Y/N)')
         Qfilter := cond
      ENDIF

   ENDIF                                         && IF nSel > 0
   Setcolor(oldcolor)
   USE
   SELECT (oldarea)
   Restscreen(,,,,savscrn)
RETURN (cond)

*----------------------------
*         Author: Ed Phillips
*   Date Created: 03/23/92
*----------------------------
FUNCTION EditQuery()
   FIELD Qfilter
   LOCAL edit_msg := " <F2> = Exit, Save Changes  <Esc> = Exit, Do Not Save the Changes "
   LOCAL savscrn := Savescreen(), cond := ''

   Alert1()
   IF Sayconfirm('Do you want to Edit the query? Y/N')
      SET FUNCTION 2 TO Chr(K_CTRL_W)
      Sayhelp(edit_msg)

      @ 17,8 TO 22,72
      @ 17,31 SAY ' Query Definition '
      Scroll(18,9,21,71,0)

      Qfilter := MemoTran(Memoedit(Trim(Qfilter),18,10,21,70,.t.,'',60), " ", " ")
      SET FUNCTION 2 TO

      Restscreen(,,,,savscrn)
      Saymemo(Qfilter,18,10,60,4)
   ENDIF

   cond := Trim(Qfilter)
RETURN(Upper(cond))

*----------------------------
*         Author: Ed Phillips
*   Date Created: 03/20/92
*----------------------------
FUNCTION DelQuery(nTop, nLeft, nBrow)
   FIELD Qnum, Qname, Quser_id

   MEMVAR uid, ex_flg
   LOCAL cond := Space(1), oldarea := Select(), nSel := 0, i := 1
   LOCAL cQnum, temp, oldcolor := Setcolor()
   LOCAL savscrn := Savescreen()

   PRIVATE aQueryList := {}

   nTop := If( ValType(nTop) = 'U', nTop := 5, nTop )
   nLeft := If(ValType(nLeft) = 'U', nLeft := 15, nLeft )
   nBrow := If( ValType(nBrow) = 'U', nBrow := 22, nBrow )

   IF Select('QUERIES') == 0
      USE Queries ALIAS Queries NEW
   ENDIF
   SELECT Queries

   GO TOP
   WHILE ! Eof()
      aAdd(aQueryList, ' '+Qname + Space(12) + Qnum)
      SKIP
   END

   WHILE .t.
      nSel = Picklist(aQueryList, "Select a Query to Delete", 50, nTop, nLeft,,nBrow)
      IF nSel > 0
         cQnum = Right(aQueryList[nSel],7)
         LOCATE FOR Qname = Subs(aQueryList[nSel],2,30) .AND. Qnum = cQnum

         IF Deleted()
            RECALL
            aQueryList[nSel] = ' '+Subs(aQueryList[nSel],2)
         ELSE
            DELETE
            aQueryList[nSel] = '*'+Subs(aQueryList[nSel],2)
         ENDIF
      ELSE
         EXIT
      ENDIF                                         && IF nSel > 0
   END
   PACK
   USE
   
   Setcolor(oldcolor)
   SELECT (oldarea)
   Restscreen(,,,,savscrn)
RETURN (NIL)
* EOF: QUERY.PRG
